/********************************************************

	Copyright 2003 Masanao Izumo <iz@onicos.co.jp>

*********************************************************/

#ifndef __LV_RESAMPLE_H__
#define __LV_RESAMPLE_H__

#define RESAMPLE_FRAC_BITS 14
#define RESAMPLE_FRAC_MASK ((1<<RESAMPLE_FRAC_BITS)-1)
#define RESAMPLE_DEFAULT_BUFSIZ	(32 * 1024)

typedef struct Resampler {
  int nch;
  int src_freq;
  int dst_freq;

  /* resampling buffer */
  int inbuf_size;
  short *inbuf;
  int inbuf_max;
  int inbuf_len;
  short *outbuf;
  int outbuf_samples;		/* in samples */

  /* Current sample counter */
  int ofs;

  /* Sample increment  (ofs += incr for each samples) */
  double f_incr;
  int incr;
  int incr_min;
  int incr_max;
  int precalc_loop_gard;

  int input_count, output_count;
} Resampler;

int resample_init(Resampler *rs, int inbuf_max_samples, int nch,
		  int src_freq, int dst_freq);
int resample_change_rate(Resampler *rs, int src_freq, int dst_freq);
void resample_free(Resampler *rs);
int resample_convert_size(Resampler *rs, int in_samples);
short *resample_linear_convert(Resampler *rs,
			       const short *inbuf, int in_samples,
			       int *out_samples);
void resample_adjust_incr(Resampler *rs);


/* Anti Aliasing */
#define ALIASING_CONVOLUTION_SIZE 64
#define ALIASING_MAX_CHANNELS 8

typedef struct AntiAliasing {
  int nch;

  /* sample buffer */
  short *inbuf;
  int inbuf_len;
  short *outbuf;
  int outbuf_size;
  int   ih[ALIASING_CONVOLUTION_SIZE/2+1];
  double fh[ALIASING_CONVOLUTION_SIZE/2+1];
} AntiAliasing;

int anti_aliasing_init(AntiAliasing *aa, int nch, int inbuf_max,
		       int src_freq, int cut_freq);
int anti_aliasing_change_rate(AntiAliasing *aa,
			      double src_freq, double cut_freq);
void anti_aliasing_free(AntiAliasing *aa);
short *anti_aliasing_convert_fast(AntiAliasing *aa,
				  const short *inbuf, int in_samples,
				  int *out_samples);
short *anti_aliasing_convert(AntiAliasing *aa,
			     const short *inbuf, int in_samples,
			     int *out_samples);


typedef struct AntiAliasingResample {
  int nch;
  AntiAliasing aa;
  Resampler rs;
} AntiAliasingResample;

int anti_aliasing_resample_init(AntiAliasingResample *aar,
				int nch, int src_freq, int dst_freq);
void anti_aliasing_resample_free(AntiAliasingResample *aar);
char *anti_aliasing_resample(AntiAliasingResample *aar,
			     const char *inbuf, int in_size, int *out_size);

int simple_up_resampling(void *pcm, int bits, int nch, int size, int ratio);

#endif
